@@ -17,6 +17,8 @@ import time
 import imp
 import re
 import atexit
+import signal
+import inspect
 
 
 import DebugProtocol
@@ -501,6 +503,7 @@ class DebugClientBase(object):
                 self.dircache = []
                 sys.argv = []
                 wd, fn, args, tracePython = arg.split('|')
+                fn = fn.encode(sys.getfilesystemencoding())
                 self.__setCoding(fn)
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
@@ -521,6 +524,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # clear all old breakpoints, they'll get set after we
                 # have started
@@ -546,6 +550,7 @@ class DebugClientBase(object):
             if cmd == DebugProtocol.RequestRun:
                 sys.argv = []
                 wd, fn, args = arg.split('|')
+                fn = fn.encode(sys.getfilesystemencoding())
                 self.__setCoding(fn)
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
@@ -566,6 +571,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 self.mainThread.tracePython = 0
                 
@@ -578,13 +584,14 @@ class DebugClientBase(object):
                     res = exc.code
                     atexit._run_exitfuncs()
                 self.writestream.flush()
-                self.progTerminated(res)
+                self.progTerminated(res, exit=True)
                 return
 
             if cmd == DebugProtocol.RequestCoverage:
                 from coverage import coverage
                 sys.argv = []
                 wd, fn, args, erase = arg.split('@@')
+                fn = fn.encode(sys.getfilesystemencoding())
                 self.__setCoding(fn)
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
@@ -597,12 +604,12 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # generate a coverage object
                 self.cover = coverage(
                     auto_data=True,
                     data_file="%s.coverage" % os.path.splitext(sys.argv[0])[0])
-                self.cover.use_cache(True)
                 
                 if int(erase):
                     self.cover.erase()
@@ -619,7 +626,7 @@ class DebugClientBase(object):
                 self.cover.stop()
                 self.cover.save()
                 self.writestream.flush()
-                self.progTerminated(res)
+                self.progTerminated(res, exit=True)
                 return
             
             if cmd == DebugProtocol.RequestProfile:
@@ -627,6 +634,7 @@ class DebugClientBase(object):
                 import PyProfile
                 sys.argv = []
                 wd, fn, args, erase = arg.split('|')
+                fn = fn.encode(sys.getfilesystemencoding())
                 self.__setCoding(fn)
                 sys.argv.append(fn)
                 sys.argv.extend(eval(args))
@@ -639,6 +647,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 # generate a profile object
                 self.prof = PyProfile.PyProfile(sys.argv[0])
@@ -656,7 +665,7 @@ class DebugClientBase(object):
                     atexit._run_exitfuncs()
                 self.prof.save()
                 self.writestream.flush()
-                self.progTerminated(res)
+                self.progTerminated(res, exit=True)
                 return
 
             if cmd == DebugProtocol.RequestShutdown:
@@ -665,6 +674,7 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestBreak:
                 fn, line, temporary, set, cond = arg.split('@@')
+                fn = fn.encode(sys.getfilesystemencoding())
                 line = int(line)
                 set = int(set)
                 temporary = int(temporary)
@@ -689,6 +699,7 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestBreakEnable:
                 fn, line, enable = arg.split(',')
+                fn = fn.encode(sys.getfilesystemencoding())
                 line = int(line)
                 enable = int(enable)
                 
@@ -703,6 +714,7 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestBreakIgnore:
                 fn, line, count = arg.split(',')
+                fn = fn.encode(sys.getfilesystemencoding())
                 line = int(line)
                 count = int(count)
                 
@@ -759,7 +771,7 @@ class DebugClientBase(object):
                 try:
                     value = eval(
                         arg, self.currentThread.getCurrentFrame().f_globals,
-                        self.currentThread.getCurrentFrameLocals())
+                        self.currentThread.getFrameLocals(0))
                 except:
                     # Report the exception and the traceback
                     try:
@@ -789,7 +801,7 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestExec:
                 _globals = self.currentThread.getCurrentFrame().f_globals
-                _locals = self.currentThread.getCurrentFrameLocals()
+                _locals = self.currentThread.getFrameLocals(0)
                 try:
                     code = compile(arg + '\n', '<stdin>', 'single')
                     exec code in _globals, _locals
@@ -842,6 +854,7 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestUTPrepare:
                 fn, tn, tfn, failed, cov, covname, erase = arg.split('|')
+                fn = fn.encode(sys.getfilesystemencoding())
                 sys.path.insert(0, os.path.dirname(os.path.abspath(fn)))
                 os.chdir(sys.path[0])
                 failed = eval(failed)
@@ -849,6 +862,7 @@ class DebugClientBase(object):
                 # set the system exception handling function to ensure, that
                 # we report on all unhandled exceptions
                 sys.excepthook = self.__unhandled_exception
+                self.__interceptSignals()
                 
                 try:
                     import unittest
@@ -878,7 +892,6 @@ class DebugClientBase(object):
                     self.cover = coverage(
                         auto_data=True,
                         data_file="%s.coverage" % os.path.splitext(covname)[0])
-                    self.cover.use_cache(True)
                     if int(erase):
                         self.cover.erase()
                 else:
@@ -966,7 +979,8 @@ class DebugClientBase(object):
                                     frmnr -= 1
                                 _globals = cf.f_globals
                                 _locals = \
-                                    self.currentThread.getCurrentFrameLocals()
+                                    self.currentThread.getFrameLocals(
+                                        self.framenr)
                         # reset sys.stdout to our redirector (unconditionally)
                         if "sys" in _globals:
                             __stdout = _globals["sys"].stdout
@@ -1129,23 +1143,11 @@ class DebugClientBase(object):
         @param redirect flag indicating redirection of stdin, stdout and
             stderr (boolean)
         """
-        if remoteAddress is None:                       # default: 127.0.0.1
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.connect((DebugProtocol.DebugAddress, port))
-        else:
-            if "@@i" in remoteAddress:
-                remoteAddress, index = remoteAddress.split("@@i")
-            else:
-                index = 0
-            if ":" in remoteAddress:                    # IPv6
-                sockaddr = socket.getaddrinfo(
-                    remoteAddress, port, 0, 0, socket.SOL_TCP)[0][-1]
-                sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-                sockaddr = sockaddr[:-1] + (int(index),)
-                sock.connect(sockaddr)
-            else:                                       # IPv4
-                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                sock.connect((remoteAddress, port))
+        if remoteAddress is None:
+            remoteAddress = "127.0.0.1"
+        elif "@@i" in remoteAddress:
+            remoteAddress = remoteAddress.split("@@i")[0]
+        sock = socket.create_connection((remoteAddress, port))
 
         self.readstream = AsyncFile(sock, sys.stdin.mode, sys.stdin.name)
         self.writestream = AsyncFile(sock, sys.stdout.mode, sys.stdout.name)
@@ -1169,6 +1171,61 @@ class DebugClientBase(object):
         @param exctb traceback for the exception
         """
         self.mainThread.user_exception(None, (exctype, excval, exctb), 1)
+    
+    def __interceptSignals(self):
+        """
+        Private method to intercept common signals.
+        """
+        for signum in [
+            signal.SIGABRT,                 # abnormal termination
+            signal.SIGFPE,                  # floating point exception
+            signal.SIGILL,                  # illegal instruction
+            signal.SIGSEGV,                 # segmentation violation
+        ]:
+            signal.signal(signum, self.__signalHandler)
+    
+    def __signalHandler(self, signalNumber, stackFrame):
+        """
+        Private method to handle signals.
+        
+        @param signalNumber number of the signal to be handled
+        @type int
+        @param stackFrame current stack frame
+        @type frame object
+        """
+        if signalNumber == signal.SIGABRT:
+            message = "Abnormal Termination"
+        elif signalNumber == signal.SIGFPE:
+            message = "Floating Point Exception"
+        elif signalNumber == signal.SIGILL:
+            message = "Illegal Instruction"
+        elif signalNumber == signal.SIGSEGV:
+            message = "Segmentation Violation"
+        else:
+            message = "Unknown Signal '%d'" % signalNumber
+        
+        filename = self.absPath(stackFrame)
+        
+        linenr = stackFrame.f_lineno
+        ffunc = stackFrame.f_code.co_name
+        
+        if ffunc == '?':
+            ffunc = ''
+        
+        if ffunc and not ffunc.startswith("<"):
+            argInfo = inspect.getargvalues(stackFrame)
+            try:
+                fargs = inspect.formatargvalues(
+                    argInfo.args, argInfo.varargs,
+                    argInfo.keywords, argInfo.locals)
+            except Exception:
+                fargs = ""
+        else:
+            fargs = ""
+        
+        siglist = [message, [filename, linenr, ffunc, fargs]]
+        
+        self.write("%s%s" % (DebugProtocol.ResponseSignal, str(siglist)))
         
     def absPath(self, fn):
         """
@@ -1243,11 +1300,13 @@ class DebugClientBase(object):
         """
         return self.running
 
-    def progTerminated(self, status):
+    def progTerminated(self, status, exit=False):
         """
         Public method to tell the debugger that the program has terminated.
         
-        @param status the return status
+        @param status return status
+        @param exit flag indicating to perform a sys.exit()
+        @type bool
         """
         if status is None:
             status = 0
@@ -1261,6 +1320,9 @@ class DebugClientBase(object):
             self.set_quit()
             self.running = None
             self.write('%s%d\n' % (DebugProtocol.ResponseExit, status))
+            if exit:
+                self.writestream.close(1)
+                sys.exit(status)
         
         # reset coding
         self.__coding = self.defaultCoding
@@ -1368,7 +1430,7 @@ class DebugClientBase(object):
                     udict = dict
                 ndict = {}
                 # this has to be in line with VariablesViewer.indicators
-                if var[i][-2:] in ["[]", "()", "{}"]:
+                if var[i][-2:] in ["[]", "()", "{}"]:   # __IGNORE_WARNING__
                     if i + 1 == len(var):
                         if var[i][:-2] == '...':
                             dictkeys = [var[i - 1]]
@@ -1394,7 +1456,7 @@ class DebugClientBase(object):
                                     oaccess = ''
                                 else:
                                     access = '%s[%s]' % (access, var[i][:-2])
-                        if var[i][-2:] == "{}":
+                        if var[i][-2:] == "{}":         # __IGNORE_WARNING__
                             isDict = 1
                         break
                     else:
@@ -1428,7 +1490,8 @@ class DebugClientBase(object):
                             exec 'mdict = dict%s.__dict__' % access
                             ndict.update(mdict)     # __IGNORE_WARNING__
                             exec 'obj = dict%s' % access
-                            if "PyQt4." in str(type(obj)):
+                            if "PyQt4." in str(type(obj)) or \
+                                    "PyQt5." in str(type(obj)):
                                 qtVariable = True
                                 qvar = obj
                                 qvtype = ("%s" % type(qvar))[1:-1]\
@@ -1438,7 +1501,7 @@ class DebugClientBase(object):
                         try:
                             exec 'mcdict = dict%s.__class__.__dict__' % access
                             ndict.update(mcdict)     # __IGNORE_WARNING__
-                            if mdict and not "sipThis" in mdict.keys():  # __IGNORE_WARNING__
+                            if mdict and "sipThis" not in mdict.keys():  # __IGNORE_WARNING__
                                 del rvar[0:2]
                                 access = ""
                         except:
@@ -1454,7 +1517,8 @@ class DebugClientBase(object):
                             ndict.update(cdict)
                             exec 'obj = dict%s' % access
                             access = ""
-                            if "PyQt4." in str(type(obj)):
+                            if "PyQt4." in str(type(obj)) or \
+                                    "PyQt5." in str(type(obj)):
                                 qtVariable = True
                                 qvar = obj
                                 qvtype = ("%s" % type(qvar))[1:-1]\
@@ -1467,7 +1531,8 @@ class DebugClientBase(object):
                             ndict.update(dict[var[i]].__class__.__dict__)
                             del rvar[0]
                             obj = dict[var[i]]
-                            if "PyQt4." in str(type(obj)):
+                            if "PyQt4." in str(type(obj)) or \
+                                    "PyQt5." in str(type(obj)):
                                 qtVariable = True
                                 qvar = obj
                                 qvtype = ("%s" % type(qvar))[1:-1]\
@@ -1484,7 +1549,8 @@ class DebugClientBase(object):
                                     pass
                             ndict.update(cdict)
                             obj = dict[var[i]]
-                            if "PyQt4." in str(type(obj)):
+                            if "PyQt4." in str(type(obj)) or \
+                                    "PyQt5." in str(type(obj)):
                                 qtVariable = True
                                 qvar = obj
                                 qvtype = ("%s" % type(qvar))[1:-1]\
@@ -1496,19 +1562,19 @@ class DebugClientBase(object):
                 i += 1
             
             if qtVariable:
-                vlist = self.__formatQt4Variable(qvar, qvtype)
+                vlist = self.__formatQtVariable(qvar, qvtype)
             elif ("sipThis" in dict.keys() and len(dict) == 1) or \
                     (len(dict) == 0 and len(udict) > 0):
                 if access:
                     exec 'qvar = udict%s' % access
                 # this has to be in line with VariablesViewer.indicators
-                elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:
+                elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:   # __IGNORE_WARNING__
                     exec 'qvar = udict["%s"][%s]' % (rvar[0][:-2], rvar[1])
                 else:
                     qvar = udict[var[-1]]
                 qvtype = ("%s" % type(qvar))[1:-1].split()[1][1:-1]
-                if qvtype.startswith("PyQt4"):
-                    vlist = self.__formatQt4Variable(qvar, qvtype)
+                if qvtype.startswith(("PyQt4", "PyQt5")):
+                    vlist = self.__formatQtVariable(qvar, qvtype)
                 else:
                     vlist = []
             else:
@@ -1517,16 +1583,16 @@ class DebugClientBase(object):
                     if access:
                         exec 'qvar = udict%s' % access
                     # this has to be in line with VariablesViewer.indicators
-                    elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:
+                    elif rvar and rvar[0][-2:] in ["[]", "()", "{}"]:   # __IGNORE_WARNING__
                         exec 'qvar = udict["%s"][%s]' % (rvar[0][:-2], rvar[1])
                     else:
                         qvar = udict[var[-1]]
                     qvtype = ("%s" % type(qvar))[1:-1].split()[1][1:-1]
-                    if qvtype.startswith("PyQt4"):
+                    if qvtype.startswith(("PyQt4", "PyQt5")):
                         qtVariable = True
                 
                 if qtVariable:
-                    vlist = self.__formatQt4Variable(qvar, qvtype)
+                    vlist = self.__formatQtVariable(qvar, qvtype)
                 else:
                     # format the dictionary found
                     if dictkeys is None:
@@ -1559,7 +1625,7 @@ class DebugClientBase(object):
         self.write('%s%s\n' % (
             DebugProtocol.ResponseVariable, unicode(varlist)))
         
-    def __formatQt4Variable(self, value, vtype):
+    def __formatQtVariable(self, value, vtype):
         """
         Private method to produce a formated output of a simple Qt4/Qt5 type.
         
@@ -1609,12 +1675,12 @@ class DebugClientBase(object):
         elif qttype == 'QColor':
             varlist.append(("name", "str", "%s" % value.name()))
             r, g, b, a = value.getRgb()
-            varlist.append(("rgb", "int", "%d, %d, %d, %d" % (r, g, b, a)))
+            varlist.append(("rgba", "int", "%d, %d, %d, %d" % (r, g, b, a)))
             h, s, v, a = value.getHsv()
-            varlist.append(("hsv", "int", "%d, %d, %d, %d" % (h, s, v, a)))
+            varlist.append(("hsva", "int", "%d, %d, %d, %d" % (h, s, v, a)))
             c, m, y, k, a = value.getCmyk()
             varlist.append(
-                ("cmyk", "int", "%d, %d, %d, %d, %d" % (c, m, y, k, a)))
+                ("cmyka", "int", "%d, %d, %d, %d, %d" % (c, m, y, k, a)))
         elif qttype == 'QDate':
             varlist.append(("", "QDate", "%s" % value.toString()))
         elif qttype == 'QTime':
@@ -1898,6 +1964,7 @@ class DebugClientBase(object):
         # set the system exception handling function to ensure, that
         # we report on all unhandled exceptions
         sys.excepthook = self.__unhandled_exception
+        self.__interceptSignals()
         
         # now start debugging
         if enableTrace:
@@ -1955,6 +2022,7 @@ class DebugClientBase(object):
         # set the system exception handling function to ensure, that
         # we report on all unhandled exceptions
         sys.excepthook = self.__unhandled_exception
+        self.__interceptSignals()
         
         # This will eventually enter a local event loop.
         # Note the use of backquotes to cause a repr of self.running. The
@@ -2152,3 +2220,4 @@ class DebugClientBase(object):
 
 #
 # eflag: FileType = Python2
+# eflag: noqa = M601, M702
