@@ -12,9 +12,11 @@ import bdb
 import os
 import atexit
 import inspect
+from inspect import CO_GENERATOR
 
 from DebugProtocol import ResponseClearWatch, ResponseClearBreak, \
     ResponseLine, ResponseSyntax, ResponseException, CallTrace
+from DebugUtilities import getargvalues, formatargvalues
 
 gRecursionLimit = 64
 
@@ -89,13 +91,23 @@ class DebugBase(bdb.Bdb):
         """
         return self.currentFrame
     
-    def getCurrentFrameLocals(self):
+    def getFrameLocals(self, frmnr=0):
         """
-        Public method to return the locals dictionary of the current frame.
+        Public method to return the locals dictionary of the current frame
+        or a frame below.
         
-        @return locals dictionary of the current frame
+        @keyparam frmnr distance of frame to get locals dictionary of. 0 is
+            the current frame (int)
+        @return locals dictionary of the frame
         """
-        return self.currentFrameLocals
+        if frmnr:
+            f = self.currentFrame
+            while f is not None and frmnr > 0:
+                f = f.f_back
+                frmnr -= 1
+            return f.f_locals
+        else:
+            return self.currentFrameLocals
     
     def step(self, traceMode):
         """
@@ -255,6 +267,9 @@ class DebugBase(bdb.Bdb):
         @exception bdb.BdbQuit raised to indicate the end of the debug session
         """
         if self.stop_here(frame) or frame == self.returnframe:
+            # Ignore return events in generator except when stepping.
+            if self.stopframe and frame.f_code.co_flags & CO_GENERATOR:
+                return self.trace_dispatch
             self.user_return(frame, arg)
             if self.quitting and not self._dbgClient.passive:
                 raise bdb.BdbQuit
@@ -270,9 +285,26 @@ class DebugBase(bdb.Bdb):
         @exception bdb.BdbQuit raised to indicate the end of the debug session
         """
         if not self.__skip_it(frame):
+            # When stepping with next/until/return in a generator frame,
+            # skip the internal StopIteration exception (with no traceback)
+            # triggered by a subiterator run with the 'yield from'
+            # statement.
+            if not (frame.f_code.co_flags & CO_GENERATOR
+                    and arg[0] is StopIteration and arg[2] is None):
+                self.user_exception(frame, arg)
+                if self.quitting:
+                    raise bdb.BdbQuit
+        # Stop at the StopIteration or GeneratorExit exception when the user
+        # has set stopframe in a generator by issuing a return command, or a
+        # next/until command at the last statement in the generator before the
+        # exception.
+        elif (self.stopframe and frame is not self.stopframe
+                and self.stopframe.f_code.co_flags & CO_GENERATOR
+                and arg[0] in (StopIteration, GeneratorExit)):
             self.user_exception(frame, arg)
             if self.quitting:
                 raise bdb.BdbQuit
+        
         return self.trace_dispatch
 
     def set_trace(self, frame=None):
@@ -559,10 +591,13 @@ class DebugBase(bdb.Bdb):
                     ffunc = ''
                 
                 if ffunc and not ffunc.startswith("<"):
-                    argInfo = inspect.getargvalues(fr)
-                    fargs = inspect.formatargvalues(
-                        argInfo.args, argInfo.varargs,
-                        argInfo.keywords, argInfo.locals)
+                    argInfo = getargvalues(fr)
+                    try:
+                        fargs = formatargvalues(
+                            argInfo.args, argInfo.varargs,
+                            argInfo.keywords, argInfo.locals)
+                    except Exception:
+                        fargs = ""
                 else:
                     fargs = ""
                 
@@ -618,10 +653,13 @@ class DebugBase(bdb.Bdb):
                     ffunc = ''
                 
                 if ffunc and not ffunc.startswith("<"):
-                    argInfo = inspect.getargvalues(fr)
-                    fargs = inspect.formatargvalues(
-                        argInfo.args, argInfo.varargs,
-                        argInfo.keywords, argInfo.locals)
+                    argInfo = getargvalues(fr)
+                    try:
+                        fargs = formatargvalues(
+                            argInfo.args, argInfo.varargs,
+                            argInfo.keywords, argInfo.locals)
+                    except Exception:
+                        fargs = ""
                 else:
                     fargs = ""
                 
@@ -722,10 +760,13 @@ class DebugBase(bdb.Bdb):
                     ffunc = ''
                 
                 if ffunc and not ffunc.startswith("<"):
-                    argInfo = inspect.getargvalues(fr)
-                    fargs = inspect.formatargvalues(
-                        argInfo.args, argInfo.varargs,
-                        argInfo.keywords, argInfo.locals)
+                    argInfo = getargvalues(fr)
+                    try:
+                        fargs = formatargvalues(
+                            argInfo.args, argInfo.varargs,
+                            argInfo.keywords, argInfo.locals)
+                    except Exception:
+                        fargs = ""
                 else:
                     fargs = ""
                 
@@ -815,7 +856,7 @@ class DebugBase(bdb.Bdb):
         if fn[0] == '<':
             return True
 
-        #XXX - think of a better way to do this.  It's only a convience for
+        #XXX - think of a better way to do this.  It's only a convenience for
         #debugging the debugger - when the debugger code is in the current
         #directory.
         if os.path.basename(fn) in [
@@ -849,3 +890,6 @@ class DebugBase(bdb.Bdb):
         @return last debugger event (string)
         """
         return self.__event
+
+#
+# eflag: noqa = M702
