@@ -24,6 +24,7 @@ from E5Gui import E5FileDialog
 from .Ui_DownloadItem import Ui_DownloadItem
 
 from .DownloadUtilities import timeString, dataString
+from ..HelpUtilities import parseContentDisposition
 
 import UI.PixmapCache
 import Preferences
@@ -269,26 +270,11 @@ class DownloadItem(QWidget, Ui_DownloadItem):
         @param directory name of the directory to store the file into (string)
         @return proposed filename and original filename (string, string)
         """
-        path = ""
-        if self.__reply.hasRawHeader("Content-Disposition"):
-            header = bytes(self.__reply.rawHeader("Content-Disposition"))\
-                .decode()
-            if header:
-                pos = header.find("filename=")
-                if pos != -1:
-                    path = header[pos + 9:]
-                    if path.startswith('"') and path.endswith('"'):
-                        path = path[1:-1]
-        if not path:
-            path = self.__url.path()
-        
+        path = parseContentDisposition(self.__reply)
         info = QFileInfo(path)
         baseName = info.completeBaseName()
         endName = info.suffix()
         
-        if not baseName:
-            baseName = "unnamed_download"
-        
         origName = baseName
         if endName:
             origName += '.' + endName
@@ -528,8 +514,11 @@ class DownloadItem(QWidget, Ui_DownloadItem):
         if self.bytesTotal() == -1:
             return -1.0
         
-        timeRemaining = (self.bytesTotal() -
-                         self.bytesReceived()) / self.currentSpeed()
+        cSpeed = self.currentSpeed()
+        if cSpeed != 0:
+            timeRemaining = (self.bytesTotal() - self.bytesReceived()) / cSpeed
+        else:
+            timeRemaining = 1
         
         # ETA should never be 0
         if timeRemaining == 0:
