@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 
+# Copyright (c) 2010 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+#
+
 """
 Module implementing the download manager class.
 """
@@ -42,6 +45,7 @@ class DownloadManager(QDialog, Ui_DownloadManager):
         """
         super(DownloadManager, self).__init__(parent)
         self.setupUi(self)
+        self.setWindowFlags(Qt.Window)
         
         self.__saveTimer = AutoSaver(self, self.save)
         
