@@ -16,7 +16,7 @@ import sys
 import pysvn
 
 from PyQt5.QtCore import QMutexLocker, QFileInfo, QDateTime, Qt, pyqtSlot
-from PyQt5.QtGui import QColor, QCursor, QBrush, QTextCursor
+from PyQt5.QtGui import QCursor, QTextCursor
 from PyQt5.QtWidgets import QWidget, QApplication, QDialogButtonBox
 
 from E5Gui.E5Application import e5App
@@ -24,6 +24,7 @@ from E5Gui import E5MessageBox, E5FileDialog
 
 from .SvnDialogMixin import SvnDialogMixin
 from .Ui_SvnDiffDialog import Ui_SvnDiffDialog
+from .SvnDiffHighlighter import SvnDiffHighlighter
 
 import Utilities
 import Preferences
@@ -44,23 +45,24 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         self.setupUi(self)
         SvnDialogMixin.__init__(self)
         
+        self.refreshButton = self.buttonBox.addButton(
+            self.tr("Refresh"), QDialogButtonBox.ActionRole)
+        self.refreshButton.setToolTip(
+            self.tr("Press to refresh the display"))
+        self.refreshButton.setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Save).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Close).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Cancel).setDefault(True)
         
+        self.searchWidget.attachTextEdit(self.contents)
+        
         self.vcs = vcs
         
         font = Preferences.getEditorOtherFonts("MonospacedFont")
         self.contents.setFontFamily(font.family())
         self.contents.setFontPointSize(font.pointSize())
         
-        self.cNormalFormat = self.contents.currentCharFormat()
-        self.cAddedFormat = self.contents.currentCharFormat()
-        self.cAddedFormat.setBackground(QBrush(QColor(190, 237, 190)))
-        self.cRemovedFormat = self.contents.currentCharFormat()
-        self.cRemovedFormat.setBackground(QBrush(QColor(237, 190, 190)))
-        self.cLineNoFormat = self.contents.currentCharFormat()
-        self.cLineNoFormat.setBackground(QBrush(QColor(255, 220, 168)))
+        self.highlighter = SvnDiffHighlighter(self.contents.document())
         
         self.client = self.vcs.getClient()
         self.client.callback_cancel = \
@@ -115,7 +117,8 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         else:
             return " "
         
-    def start(self, fn, versions=None, urls=None, summary=False, pegRev=None):
+    def start(self, fn, versions=None, urls=None, summary=False, pegRev=None,
+              refreshable=False):
         """
         Public slot to start the svn diff command.
         
@@ -126,7 +129,10 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         @keyparam summary flag indicating a summarizing diff
             (only valid for URL diffs) (boolean)
         @keyparam pegRev revision number the filename is valid (integer)
+        @keyparam refreshable flag indicating a refreshable diff (boolean)
         """
+        self.refreshButton.setVisible(refreshable)
+        
         self.buttonBox.button(QDialogButtonBox.Save).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Close).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Cancel).setEnabled(True)
@@ -267,11 +273,9 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         self.__finish()
         
         if self.paras == 0:
-            self.contents.insertPlainText(
-                self.tr('There is no difference.'))
-            return
+            self.contents.setPlainText(self.tr('There is no difference.'))
         
-        self.buttonBox.button(QDialogButtonBox.Save).setEnabled(True)
+        self.buttonBox.button(QDialogButtonBox.Save).setEnabled(self.paras > 0)
         
     def __appendText(self, line):
         """
@@ -279,21 +283,9 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         
         @param line line of text to insert (string)
         """
-        if line.startswith('+') or line.startswith('>') or \
-                line.startswith('A '):
-            format = self.cAddedFormat
-        elif line.startswith('-') or line.startswith('<') or \
-                line.startswith('D '):
-            format = self.cRemovedFormat
-        elif line.startswith('@@'):
-            format = self.cLineNoFormat
-        else:
-            format = self.cNormalFormat
-        
         tc = self.contents.textCursor()
         tc.movePosition(QTextCursor.End)
         self.contents.setTextCursor(tc)
-        self.contents.setCurrentCharFormat(format)
         self.contents.insertPlainText(line)
         self.paras += 1
         
@@ -328,8 +320,10 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
         """
         QApplication.restoreOverrideCursor()
         
+        self.refreshButton.setEnabled(True)
         self.buttonBox.button(QDialogButtonBox.Cancel).setEnabled(False)
         self.buttonBox.button(QDialogButtonBox.Close).setEnabled(True)
+        self.buttonBox.button(QDialogButtonBox.Close).setEnabled(True)
         self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
         
         tc = self.contents.textCursor()
@@ -360,7 +354,9 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
             self.__finish()
         elif button == self.buttonBox.button(QDialogButtonBox.Save):
             self.on_saveButton_clicked()
-        
+        elif button == self.refreshButton:
+            self.on_refreshButton_clicked()
+    
     @pyqtSlot(int)
     def on_filesCombo_activated(self, index):
         """
@@ -455,6 +451,18 @@ class SvnDiffDialog(QWidget, SvnDialogMixin, Ui_SvnDiffDialog):
                     '<br>Reason: {1}</p>')
                 .format(fname, str(why)))
         
+    @pyqtSlot()
+    def on_refreshButton_clicked(self):
+        """
+        Private slot to refresh the display.
+        """
+        self.buttonBox.button(QDialogButtonBox.Close).setEnabled(False)
+        
+        self.buttonBox.button(QDialogButtonBox.Save).setEnabled(False)
+        self.refreshButton.setEnabled(False)
+        
+        self.start(self.filename, refreshable=True)
+    
     def __showError(self, msg):
         """
         Private slot to show an error message.
