@@ -17,7 +17,7 @@ import os
 import sys
 import shutil
 
-from PyQt5.QtCore import QThread, QFileInfo, pyqtSignal, QProcess
+from PyQt5.QtCore import PYQT_VERSION, QThread, QFileInfo, pyqtSignal, QProcess
 from PyQt5.QtWidgets import QDialog, QInputDialog, QApplication, QMenu
 
 from E5Gui.E5Application import e5App
@@ -131,7 +131,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         
         self.menu = QMenu(self)
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             self.menu.addAction(
                 self.tr('Compile form'), self.__compileForm)
             self.menu.addAction(
@@ -186,7 +186,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         self.menuActions.append(act)
         self.menu.addSeparator()
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             self.menu.addAction(self.tr('New form...'), self.__newForm)
         else:
             if self.hooks["newForm"] is not None:
@@ -209,7 +209,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
 
         self.backMenu = QMenu(self)
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"] or \
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"] or \
                 self.hooks["compileAllForms"] is not None:
             self.backMenu.addAction(
                 self.tr('Compile all forms'), self.__compileAllForms)
@@ -236,7 +236,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         # create the menu for multiple selected files
         self.multiMenu = QMenu(self)
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             act = self.multiMenu.addAction(
                 self.tr('Compile forms'), self.__compileSelectedForms)
             self.multiMenu.addSeparator()
@@ -278,7 +278,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
 
         self.dirMenu = QMenu(self)
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             self.dirMenu.addAction(
                 self.tr('Compile all forms'), self.__compileAllForms)
             self.dirMenu.addSeparator()
@@ -298,7 +298,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         self.dirMenuActions.append(act)
         self.dirMenu.addSeparator()
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             self.dirMenu.addAction(self.tr('New form...'), self.__newForm)
         else:
             if self.hooks["newForm"] is not None:
@@ -323,7 +323,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         
         self.dirMultiMenu = QMenu(self)
         if self.project.getProjectType() in \
-                ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
             self.dirMultiMenu.addAction(
                 self.tr('Compile all forms'), self.__compileAllForms)
             self.dirMultiMenu.addSeparator()
@@ -541,7 +541,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
             self.hooks["newForm"](path)
         else:
             if self.project.getProjectType() in \
-                    ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+                    ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
                 self.__newUiForm(path)
         
     def __newUiForm(self, path):
@@ -749,18 +749,27 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
         
         if self.project.pdata["PROGLANGUAGE"][0] in \
                 ["Python", "Python2", "Python3"]:
-            if self.project.getProjectType() in ["Qt4", "E4Plugin"]:
+            if self.project.getProjectType() in ["Qt4", ]:
                 self.uicompiler = 'pyuic4'
                 if Utilities.isWindowsPlatform():
                     uic = self.uicompiler + '.bat'
                 else:
                     uic = self.uicompiler
-            elif self.project.getProjectType() in ["PyQt5", "E6Plugin"]:
+            elif self.project.getProjectType() in ["PyQt5"]:
                 self.uicompiler = 'pyuic5'
                 if Utilities.isWindowsPlatform():
                     uic = self.uicompiler + '.bat'
                 else:
                     uic = self.uicompiler
+            elif self.project.getProjectType() in ["E6Plugin"]:
+                if PYQT_VERSION < 0x050000:
+                    self.uicompiler = 'pyuic4'
+                else:
+                    self.uicompiler = 'pyuic5'
+                if Utilities.isWindowsPlatform():
+                    uic = self.uicompiler + '.bat'
+                else:
+                    uic = self.uicompiler
             elif self.project.getProjectType() == "PySide":
                 self.uicompiler = 'pyside-uic'
                 uic = Utilities.generatePySideToolPath(self.uicompiler)
@@ -937,7 +946,7 @@ class ProjectFormsBrowser(ProjectBaseBrowser):
             self.hooks["compileChangedForms"](self.project.pdata["FORMS"])
         else:
             if self.project.getProjectType() not in \
-               ["Qt4", "PyQt5", "E4Plugin", "E6Plugin", "PySide"]:
+               ["Qt4", "PyQt5", "E6Plugin", "PySide"]:
                 # ignore the request for non Qt GUI projects
                 return
             
