@@ -9,7 +9,7 @@ Module implementing the search and replace widget.
 
 from __future__ import unicode_literals
 
-from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot
+from PyQt5.QtCore import pyqtSignal, Qt, pyqtSlot, QEvent
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QToolButton, QScrollArea, \
     QSizePolicy, QFrame
 
@@ -118,7 +118,8 @@ and so on.</td></tr>
 <td>This matches 1 or more times. For example, <code>Sa+m</code> matches
 <code>Sam</code>, <code>Saam</code>, <code>Saaam</code> and so on.</td></tr>
 </table>
-""")                                                # __IGNORE_WARNING__
+"""
+        )
         self.setWhatsThis(whatsThis)
         
         self.ui.closeButton.setIcon(UI.PixmapCache.getIcon("close.png"))
@@ -159,12 +160,23 @@ and so on.</td></tr>
         self.findPrevAct.setEnabled(False)
         self.ui.findtextCombo.addAction(self.findPrevAct)
         
+        self.adjustSize()
+        
         self.havefound = False
         self.__pos = None
         self.__findBackwards = False
         self.__selections = []
         self.__finding = False
     
+    def changeEvent(self, evt):
+        """
+        Protected method handling state changes.
+        
+        @param evt event containing the state change (QEvent)
+        """
+        if evt.type() == QEvent.FontChange:
+            self.adjustSize()
+    
     def __selectionBoundary(self, selections=None):
         """
         Private method to calculate the current selection boundary.
@@ -339,6 +351,7 @@ and so on.</td></tr>
             if indicate:
                 aw.setSearchIndicator(tgtPos, tgtLen)
             ok = aw.findNextTarget()
+        aw.updateMarkerMap()
     
     def __findNextPrev(self, txt, backwards):
         """
@@ -888,7 +901,6 @@ class SearchReplaceSlidingWidget(QWidget):
         
         self.__searchReplaceWidget = \
             SearchReplaceWidget(replace, vm, self, True)
-        srHeight = self.__searchReplaceWidget.height()
         
         self.__layout = QHBoxLayout(self)
         self.setLayout(self.__layout)
@@ -905,7 +917,6 @@ class SearchReplaceSlidingWidget(QWidget):
         self.__scroller.setWidget(self.__searchReplaceWidget)
         self.__scroller.setSizePolicy(
             QSizePolicy.Expanding, QSizePolicy.Minimum)
-        self.__scroller.setMaximumHeight(srHeight)
         self.__scroller.setFrameShape(QFrame.NoFrame)
         self.__scroller.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.__scroller.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
@@ -921,13 +932,25 @@ class SearchReplaceSlidingWidget(QWidget):
         self.__layout.addWidget(self.__scroller)
         self.__layout.addWidget(self.__rightButton)
         
-        self.setMaximumHeight(srHeight)
+        self.setMaximumHeight(self.__searchReplaceWidget.sizeHint().height())
+        self.adjustSize()
         
         self.__searchReplaceWidget.searchListChanged.connect(
             self.searchListChanged)
         self.__leftButton.clicked.connect(self.__slideLeft)
         self.__rightButton.clicked.connect(self.__slideRight)
     
+    def changeEvent(self, evt):
+        """
+        Protected method handling state changes.
+
+        @param evt event containing the state change (QEvent)
+        """
+        if evt.type() == QEvent.FontChange:
+            self.setMaximumHeight(
+                self.__searchReplaceWidget.sizeHint().height())
+            self.adjustSize()
+    
     def findNext(self):
         """
         Public slot to find the next occurrence of text.
